/*
 * bamf-application-private.h
 * This file is part of BAMF
 *
 * Copyright (C) 2012 - Marco Trevisan (Treviño)
 *
 * BAMF is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * BAMF is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BAMF; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 */

#ifndef _BAMF_APPLICATION_PRIVATE_H_
#define _BAMF_APPLICATION_PRIVATE_H_

#include <libbamf/bamf-application.h>

GList * _bamf_application_get_cached_xids (BamfApplication *app);

#endif
